@@ -380,22 +380,6 @@ __glXDRIbindTexImage(__GLXcontext *baseContext,
     __GLXDRIscreen * const screen =
 	(__GLXDRIscreen *) __glXgetActiveScreen(pScreen->myNum);
 
-    if (!screen->texOffsetStart) {
-         /* When the GLX_EXT_texture_from_pixmap is used, as it's
-          * implemented here, we want to pull pixmap out of video memory
-          * and into host memory. */
-         extern void XAAEvictPixmaps(void);
-         static int evictedPixmaps = 0;
-
-         if (!evictedPixmaps) {
-             __glXDRIenterServer(GL_FALSE);
-                 if (dlsym(RTLD_DEFAULT, "XAAEvictPixmaps"))
-                      XAAEvictPixmaps();
-             __glXDRIleaveServer(GL_FALSE);
-         evictedPixmaps = TRUE;
-         }
-    }
-
     pixmap = (PixmapPtr) glxPixmap->pDraw;
 
     if (screen->texOffsetStart && screen->driScreen.setTexOffset) {
