@@ -374,7 +374,6 @@ ProcXvQueryAdaptors(ClientPtr client)
   xvAdaptorInfo ainfo;
   xvQueryAdaptorsReply rep;
   int totalSize, na, nf, rc;
-  int nameSize;
   XvAdaptorPtr pa;
   XvFormatPtr pf;
   WindowPtr pWin;
@@ -436,12 +435,12 @@ ProcXvQueryAdaptors(ClientPtr client)
       ainfo.base_id = pa->base_id;
       ainfo.num_ports = pa->nPorts;
       ainfo.type = pa->type;
-      ainfo.name_size = nameSize = strlen(pa->name);
+      ainfo.name_size = strlen(pa->name);
       ainfo.num_formats = pa->nFormats;
 
       _WriteAdaptorInfo(client, &ainfo);
 
-      WriteToClient(client, nameSize, pa->name);
+      WriteToClient(client, ainfo.name_size, pa->name);
 
       nf = pa->nFormats;
       pf = pa->pFormats;
@@ -467,7 +466,6 @@ ProcXvQueryEncodings(ClientPtr client)
   xvEncodingInfo einfo;
   xvQueryEncodingsReply rep;
   int totalSize;
-  int nameSize;
   XvPortPtr pPort;
   int ne;
   XvEncodingPtr pe;
@@ -512,13 +510,13 @@ ProcXvQueryEncodings(ClientPtr client)
   while (ne--) 
     {
       einfo.encoding = pe->id;
-      einfo.name_size = nameSize = strlen(pe->name);
+      einfo.name_size = strlen(pe->name);
       einfo.width = pe->width;
       einfo.height = pe->height;
       einfo.rate.numerator = pe->rate.numerator;
       einfo.rate.denominator = pe->rate.denominator;
       _WriteEncodingInfo(client, &einfo);
-      WriteToClient(client, nameSize, pe->name);
+      WriteToClient(client, einfo.name_size, pe->name);
       pe++;
     }
 
@@ -996,19 +994,18 @@ ProcXvQueryPortAttributes(ClientPtr client)
   rep.text_size = 0;
 
   for(i = 0, pAtt = pPort->pAdaptor->pAttributes; 
-      i < pPort->pAdaptor->nAttributes; i++, pAtt++) 
+      i < rep.num_attributes; i++, pAtt++) 
   {    
       rep.text_size += (strlen(pAtt->name) + 1 + 3) & ~3L;
   }
 
-  rep.length = rep.length = (pPort->pAdaptor->nAttributes * sz_xvAttributeInfo)
-      + rep.text_size;
+  rep.length = (rep.num_attributes * sz_xvAttributeInfo) + rep.text_size;
   rep.length >>= 2;
 
   _WriteQueryPortAttributesReply(client, &rep);
 
   for(i = 0, pAtt = pPort->pAdaptor->pAttributes; 
-      i < pPort->pAdaptor->nAttributes; i++, pAtt++) 
+      i < rep.num_attributes; i++, pAtt++) 
   {
       size = strlen(pAtt->name) + 1;  /* pass the NULL */
       Info.flags = pAtt->flags;
@@ -1219,7 +1216,6 @@ ProcXvQueryImageAttributes(ClientPtr client)
   XvPortPtr pPort;
   int *offsets;
   int *pitches;
-  int planeLength;
   REQUEST(xvQueryImageAttributesReq);
 
   REQUEST_SIZE_MATCH(xvQueryImageAttributesReq);
@@ -1259,7 +1255,7 @@ ProcXvQueryImageAttributes(ClientPtr client)
 
   rep.type = X_Reply;
   rep.sequenceNumber = client->sequence;
-  rep.length = planeLength = num_planes << 1;
+  rep.length = num_planes << 1;
   rep.num_planes = num_planes;
   rep.width = width;
   rep.height = height;
@@ -1267,8 +1263,8 @@ ProcXvQueryImageAttributes(ClientPtr client)
  
   _WriteQueryImageAttributesReply(client, &rep);
   if(client->swapped)
-    SwapLongs((CARD32*)offsets, planeLength);
-  WriteToClient(client, planeLength << 2, (char*)offsets);
+    SwapLongs((CARD32*)offsets, rep.length);
+  WriteToClient(client, rep.length << 2, (char*)offsets);
 
   xfree(offsets);
 
@@ -1296,13 +1292,13 @@ ProcXvListImageFormats(ClientPtr client)
   rep.type = X_Reply;
   rep.sequenceNumber = client->sequence;
   rep.num_formats = pPort->pAdaptor->nImages;
-  rep.length = pPort->pAdaptor->nImages * sz_xvImageFormatInfo >> 2;
+  rep.length = rep.num_formats * sz_xvImageFormatInfo >> 2;
 
   _WriteListImageFormatsReply(client, &rep);
 
   pImage = pPort->pAdaptor->pImages;
   
-  for(i = 0; i < pPort->pAdaptor->nImages; i++, pImage++) {
+  for(i = 0; i < rep.num_formats; i++, pImage++) {
      info.id = pImage->id; 	
      info.type = pImage->type; 	
      info.byte_order = pImage->byte_order; 
@@ -1501,7 +1497,6 @@ SProcXvShmPutImage(ClientPtr client)
   swapl(&stuff->gc, n);
   swapl(&stuff->shmseg, n);
   swapl(&stuff->id, n);
-  swapl(&stuff->offset, n);
   swaps(&stuff->src_x, n);
   swaps(&stuff->src_y, n);
   swaps(&stuff->src_w, n);
@@ -1510,6 +1505,7 @@ SProcXvShmPutImage(ClientPtr client)
   swaps(&stuff->drw_y, n);
   swaps(&stuff->drw_w, n);
   swaps(&stuff->drw_h, n);
+  swaps(&stuff->offset, n);
   swaps(&stuff->width, n);
   swaps(&stuff->height, n);
   return ProcXvShmPutImage(client);
@@ -1556,7 +1552,6 @@ SProcXvSetPortAttribute(ClientPtr client)
   swaps(&stuff->length, n);
   swapl(&stuff->port, n);
   swapl(&stuff->attribute, n);
-  swapl(&stuff->value, n);
   return ProcXvSetPortAttribute(client);
 }
 
@@ -1601,10 +1596,9 @@ SProcXvQueryImageAttributes(ClientPtr client)
   register char n;
   REQUEST(xvQueryImageAttributesReq);
   swaps(&stuff->length, n);
-  swapl(&stuff->port, n);
   swapl(&stuff->id, n);
   swaps(&stuff->width, n);
-  swaps(&stuff->height, n);
+  swaps(&stuff->width, n);
   return ProcXvQueryImageAttributes(client);
 }
 
@@ -1631,7 +1625,7 @@ SWriteQueryExtensionReply(
   swaps(&rep->version, n);
   swaps(&rep->revision, n);
   
-  (void)WriteToClient(client, sz_xvQueryExtensionReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryExtensionReply, (char *)&rep);
 
   return Success;
 }
@@ -1647,7 +1641,7 @@ SWriteQueryAdaptorsReply(
   swapl(&rep->length, n);
   swaps(&rep->num_adaptors, n);
   
-  (void)WriteToClient(client, sz_xvQueryAdaptorsReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryAdaptorsReply, (char *)&rep);
 
   return Success;
 }
@@ -1663,7 +1657,7 @@ SWriteQueryEncodingsReply(
   swapl(&rep->length, n);
   swaps(&rep->num_encodings, n);
   
-  (void)WriteToClient(client, sz_xvQueryEncodingsReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryEncodingsReply, (char *)&rep);
 
   return Success;
 }
@@ -1770,7 +1764,7 @@ SWriteGrabPortReply(
   swaps(&rep->sequenceNumber, n);
   swapl(&rep->length, n);
 
-  (void)WriteToClient(client, sz_xvGrabPortReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvGrabPortReply, (char *)&rep);
 
   return Success;
 }
@@ -1786,7 +1780,7 @@ SWriteGetPortAttributeReply(
   swapl(&rep->length, n);
   swapl(&rep->value, n);
 
-  (void)WriteToClient(client, sz_xvGetPortAttributeReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvGetPortAttributeReply, (char *)&rep);
 
   return Success;
 }
@@ -1803,7 +1797,7 @@ SWriteQueryBestSizeReply(
   swaps(&rep->actual_width, n);
   swaps(&rep->actual_height, n);
 
-  (void)WriteToClient(client, sz_xvQueryBestSizeReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryBestSizeReply, (char *)&rep);
 
   return Success;
 }
@@ -1820,7 +1814,7 @@ SWriteQueryPortAttributesReply(
   swapl(&rep->num_attributes, n);
   swapl(&rep->text_size, n);
 
-  (void)WriteToClient(client, sz_xvQueryPortAttributesReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryPortAttributesReply, (char *)&rep);
 
   return Success;
 }
@@ -1839,7 +1833,7 @@ SWriteQueryImageAttributesReply(
   swaps(&rep->width, n);
   swaps(&rep->height, n);
 
-  (void)WriteToClient(client, sz_xvQueryImageAttributesReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvQueryImageAttributesReply, (char *)&rep);
 
   return Success;
 }
@@ -1856,7 +1850,7 @@ SWriteListImageFormatsReply(
   swapl(&rep->length, n);
   swapl(&rep->num_formats, n);
 
-  (void)WriteToClient(client, sz_xvListImageFormatsReply, (char *)rep);
+  (void)WriteToClient(client, sz_xvListImageFormatsReply, (char *)&rep);
 
   return Success;
 }
