@@ -77,10 +77,10 @@ void FreeAtom(NodePtr patom);
 _X_EXPORT Atom 
 MakeAtom(char *string, unsigned len, Bool makeit)
 {
-    register    NodePtr * np;
+    NodePtr * np;
     unsigned i;
-    int     comp;
-    register unsigned int   fp = 0;
+    int comp;
+    unsigned int fp = 0;
 
     np = &atomRoot;
     for (i = 0; i < (len+1)/2; i++)
@@ -107,7 +107,7 @@ MakeAtom(char *string, unsigned len, Bool makeit)
     }
     if (makeit)
     {
-	register NodePtr nd;
+	NodePtr nd;
 
 	nd = (NodePtr) xalloc(sizeof(NodeRec));
 	if (!nd)
@@ -167,7 +167,7 @@ NameForAtom(Atom atom)
 }
 
 void
-AtomError()
+AtomError(void)
 {
     FatalError("initializing atoms");
 }
@@ -185,7 +185,7 @@ FreeAtom(NodePtr patom)
 }
 
 void
-FreeAllAtoms()
+FreeAllAtoms(void)
 {
     if(atomRoot == (NodePtr)NULL)
 	return;
@@ -197,7 +197,7 @@ FreeAllAtoms()
 }
 
 void
-InitAtoms()
+InitAtoms(void)
 {
     FreeAllAtoms();
     tableLength = InitialTableSize;
