@@ -30,6 +30,10 @@
  * use or other dealings in this Software without prior written authorization.
  */
 
+#ifdef HAVE_XORG_CONFIG_H
+#include <xorg-config.h>
+#endif
+
 #include <X11/X.h>
 #include <X11/Xproto.h>
 #include "os.h"
@@ -200,6 +204,12 @@ DarwinPrintBanner()
 }
 
 
+void DDXRingBell(int volume, int pitch, int duration)
+{
+  // FIXME -- make some noise, yo
+}
+
+
 /*
  * DarwinSaveScreen
  *  X screensaver support. Not implemented.
@@ -409,9 +419,9 @@ static int DarwinMouseProc(
             InitPointerDeviceStruct( (DevicePtr)pPointer,
                         map,
                         5,   // numbuttons (4 & 5 are scroll wheel)
-                        miPointerGetMotionEvents,
+                        GetMotionHistory,
                         DarwinChangePointerControl,
-                        0 );
+                        GetMotionHistorySize(), 2 );
 #ifdef XINPUT
             InitValuatorAxisStruct( pPointer,
                                     0,     // X axis
