@@ -30,6 +30,8 @@
 
 extern Window EphyrPreExistingHostWin;
 extern Bool   EphyrWantGrayScale;
+extern Bool   kdHasPointer;
+extern Bool   kdHasKbd;
 
 void
 InitCard (char *name)
@@ -45,6 +47,9 @@ InitCard (char *name)
 void
 InitOutput (ScreenInfo *pScreenInfo, int argc, char **argv)
 {
+#ifdef GLXEXT
+  noGlxExtension=TRUE;
+#endif
   KdInitOutput (pScreenInfo, argc, argv);
 }
 
@@ -54,19 +59,30 @@ InitInput (int argc, char **argv)
   KdKeyboardInfo *ki;
   KdPointerInfo *pi;
         
-  ki = KdNewKeyboard();
-  if (!ki)
-    FatalError("Couldn't create Xephyr keyboard\n");
-  ki->driver = &EphyrKeyboardDriver;
   KdAddKeyboardDriver(&EphyrKeyboardDriver);
-  KdAddKeyboard(ki);
-
-  pi = KdNewPointer();
-  if (!pi)
-    FatalError("Couldn't create Xephyr pointer\n");
-  pi->driver = &EphyrMouseDriver;
+#ifdef linux
+  KdAddKeyboardDriver(&LinuxEvdevKeyboardDriver);
+#endif
   KdAddPointerDriver(&EphyrMouseDriver);
-  KdAddPointer(pi);
+#ifdef linux
+  KdAddPointerDriver(&LinuxEvdevMouseDriver);
+#endif
+
+  if (!kdHasKbd) {
+    ki = KdNewKeyboard();
+    if (!ki)
+      FatalError("Couldn't create Xephyr keyboard\n");
+    ki->driver = &EphyrKeyboardDriver;
+    KdAddKeyboard(ki);
+  }
+
+  if (!kdHasPointer) {
+    pi = KdNewPointer();
+    if (!pi)
+      FatalError("Couldn't create Xephyr pointer\n");
+    pi->driver = &EphyrMouseDriver;
+    KdAddPointer(pi);
+  }
 
   KdInitInput();
 }
