@@ -48,6 +48,7 @@
 	"\tLoad\t\"dbe\"\n" \
 	"\tLoad\t\"glx\"\n" \
 	"\tLoad\t\"freetype\"\n" \
+	"\tLoad\t\"type1\"\n" \
 	"\tLoad\t\"record\"\n" \
 	"\tLoad\t\"dri\"\n" \
 	"EndSection\n\n"
@@ -159,12 +160,17 @@ videoPtrToDriverName(pciVideoPtr info)
 {
     /*
      * things not handled yet:
-     * amd/cyrix/nsc
-     * xgi
+     * cyrix/nsc.  should be merged into geode anyway.
+     * xgi.
      */
 
     switch (info->vendor)
     {
+	case 0x1022:
+		if (info->chipType == 0x2081)
+			return "amd";
+		else
+			return NULL;
 	case 0x1142:		    return "apm";
 	case 0xedd8:		    return "ark";
 	case 0x1a03:		    return "ast";
