@@ -39,10 +39,6 @@
 #include <xorg-config.h>
 #endif
 
-#ifdef __UNIXOS2__
-#define I_NEED_OS2_H
-#endif
-
 #include <misc.h>
 #include <xf86.h>
 #include <xf86Version.h>
