@@ -3,7 +3,6 @@
 
    Written by Mark Vojkovich
 */
-/* $XFree86: xc/programs/Xserver/Xext/xf86dga2.c,v 1.17 2001/10/28 03:32:51 tsi Exp $ */
 
 
 #ifdef HAVE_XORG_CONFIG_H
@@ -30,11 +29,7 @@
 #include "dgaproc.h"
 #include "xf86dgaext.h"
 
-#ifdef EXTMODULE
-#include "xf86_ansic.h"
-#else
 #include <string.h>
-#endif
 
 #include "modinit.h"
 
@@ -766,10 +761,8 @@ ProcXDGADispatch (ClientPtr client)
     }
 }
 
-#ifdef EXTMODULE
 void
 XFree86DGARegister(INITARGS)
 {
   XDGAEventBase = &DGAEventBase; 
 }
-#endif
