@@ -92,30 +92,6 @@ XAADestroyInfoRec(XAAInfoRecPtr infoRec)
     xfree(infoRec);
 }
 
-void
-XAAEvictPixmaps(void)
-{
-    XAAScreenPtr pScreenPriv;
-    XAAInfoRecPtr infoRec;
-    ScreenPtr pScreen;
-    int i;
-
-    xf86MsgVerb(X_INFO, 3, "XAA: Evicting pixmaps\n");
-
-    for (i = 0; i < screenInfo.numScreens; i++) {
-	pScreen = screenInfo.screens[i];
-	infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
-
-	pScreenPriv = pScreen->devPrivates[XAAScreenIndex].ptr;
-	infoRec = pScreenPriv->AccelInfoRec;
-
-	infoRec->offscreenDepths = 0;
-	infoRec->Flags &= ~OFFSCREEN_PIXMAPS;
-       
-	XAAMoveOutOffscreenPixmaps(pScreen);
-	XAAInvalidatePixmapCache(pScreen);
-    }
-}
 
 Bool 
 XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
