@@ -1,3 +1,3 @@
 #define OPEQ MFB_OPEQ_INVERT
 #define MFBPOLYGLYPHBLT mfbPolyGlyphBltInvert
-#include "./mfbplygblt.c"
+#include "../../mfb/mfbplygblt.c"
