@@ -1,4 +1,4 @@
 #define OPEQ MFB_OPEQ_BLACK
 #define MFBFILLPOLY1RECT mfbFillPolyBlack
 #define EQWHOLEWORD MFB_EQWHOLEWORD_BLACK
-#include "./mfbply1rct.c"
+#include "../../mfb/mfbply1rct.c"
