@@ -1,4 +1,4 @@
 #define OPEQ MFB_OPEQ_WHITE
 #define MFBFILLPOLY1RECT mfbFillPolyWhite
 #define EQWHOLEWORD MFB_EQWHOLEWORD_WHITE
-#include "./mfbply1rct.c"
+#include "../../mfb/mfbply1rct.c"
