@@ -89,6 +89,7 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #include <stdlib.h>	/* for malloc() */
 #include <errno.h>
 
+#include "input.h"
 #include "site.h"
 #include "opaque.h"
 
@@ -181,9 +182,6 @@ LogInit(const char *fname, const char *backup)
 		sprintf(suffix, backup, display);
 		sprintf(oldLog, "%s%s", logFileName, suffix);
 		free(suffix);
-#ifdef __UNIXOS2__
-		remove(oldLog);
-#endif
 		if (rename(logFileName, oldLog) == -1) {
 		    FatalError("Cannot move old log file (\"%s\" to \"%s\"\n",
 			       logFileName, oldLog);
@@ -220,7 +218,7 @@ LogInit(const char *fname, const char *backup)
 }
 
 void
-LogClose()
+LogClose(void)
 {
     if (logFile) {
 	fclose(logFile);
@@ -404,6 +402,7 @@ void
 AbortServer(void)
 {
     OsCleanup(TRUE);
+    CloseDownDevices();
     AbortDDX();
     fflush(stderr);
     if (CoreDump)
@@ -614,7 +613,7 @@ Error(char *str)
 }
 
 void
-LogPrintMarkers()
+LogPrintMarkers(void)
 {
     /* Show what the message marker symbols mean. */
     ErrorF("Markers: ");
